/*****************************************************************************
 *
 * - add.c - 
 *
 * Implements "L.Add" action for adding entries. The add entry dialog
 * is doubles for the modify dialog, except operating in "add" mode.
 *
 ****************************************************************************/

#include "defs.h"
#include "add.h"
#include "modify.h"
#include "objects.h"
#include "update1.h"
#include "update2.h"
#include "xutil.h"
#include "requests.h"
#include "y.tab.h"

#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/MenuShell.h>
#include <Xm/List.h>
#include <Xm/Label.h>
#include <Xm/TextF.h>

extern XLUentryClass entry_class_list;
extern Display *dpy;

/**************************************************
 *
 * Local functions..
 * 
 */

/* Add dialog get\'s entry\'s RDN, parent and entry class */
static void addDialog();	
static void addOk();	   /* Callback for OK button on add dialog */
static void addCancel();   /* Similarly... */
static void selectClass(); /* Callback for list box selection in add dialog */
static void checkEntryName();
static void gotNameCheckResult();

/***************************************************
 *
 * Global storage.
 *
 */

static Widget add_dialog = NULLWidget;
static Widget ADDpw; 
static int ADDobjid;
static int ADDprimid;
static int ADDactid;
static XLUactionList ADDactlist;
static XLUentryClass ADDclass;
static String ADDrdn = NULLString;

/*****************************************************************************
 * doAddAction()
 *
 * Find the parent entry to add under, the RDN of the new entry and the
 * class of entry to add.
 *
 ****************************************************************************/

/* ARGSUSED */
void doAddAction(pw, objid, primid, actid, actlist)
     Widget pw;
     int objid;
     int primid;
     int actid;
     XLUactionList actlist;
{
  XLUwindowObject winobj;
  String dname;

  ADDpw = pw;
  ADDobjid = objid;
  ADDprimid = primid;
  ADDactid = actid;
  ADDactlist = actlist;

  /* The default parent name is the current entry of the given window */
  winobj = XLUgetWindowObject(objid);
  if (winobj != (XLUwindowObject) NULL &&
      winobj->current_entry != NULLEntryList)
    dname = winobj->current_entry->string_dn;
  else
    dname = "";

  addDialog(objid, dname);
} /* doAddAction */

/*****************************************************************************
 * addDialog()
 *
 * Query user for name, parent of and type of entry to be added.
 *
 ****************************************************************************/

/* ARGSUSED */
static void addDialog(objid, parentdn)
     int objid;
     String parentdn;
{
  Arg args[MAXARGS];
  int count, strcount;
  Widget popup_shell, form;
  Widget entry_name, parent_name, type_name;
  Widget ok_button, cancel_button, label, list;
  XLUentryClass class;
  XmString *class_strings;
  Position x, y;
  Dimension width, height;
  char *name = "Xlookup - Add a new entry";
  XTextProperty windowName, iconName;
  XSizeHints size_hints;
  XWMHints wm_hints;
  XClassHint class_hints;
  BOOLEAN changes;
  XLUupdateState state;
  
  extern Widget toplevel;
  extern String resource_class, program_name;
  
  /***************************************************
   *
   * Make the dialog.
   *
   */
  
  add_dialog = popup_shell = XtCreatePopupShell("addDialogShell",
						transientShellWidgetClass,
						toplevel,
						args,
						0);

  XtSetArg(args[0], XmNverticalSpacing, 10);
  XtSetArg(args[1], XmNhorizontalSpacing, 10);

  form = XmCreateForm(popup_shell, "addDialogForm", args, 2);
  XtManageChild(form);

  /**************************************************
   *
   * OK and Cancel buttons.
   * 
   */
  
  ok_button = XmCreatePushButton(form, "addOkButton", args, 0);
  XtManageChild(ok_button);
  XtAddCallback(ok_button, XmNactivateCallback, addOk, (XtPointer) objid);
  
  XtSetArg(args[0], XmNleftWidget, ok_button);
  cancel_button = XmCreatePushButton(form, "addCancelButton", args, 1);

  XtManageChild(cancel_button);
  XtAddCallback(cancel_button,
		XmNactivateCallback,
		addCancel,
		(XtPointer) objid);

  /**************************************************
   *
   * Entry name edit field and label.
   * 
   */

  XtSetArg(args[0], XmNtopWidget, ok_button);
  label = XmCreateLabel(form, "addEntryNameLabel", args, 1);

  XtManageChild(label);

  XtSetArg(args[0], XmNtopWidget, ok_button);
  XtSetArg(args[1], XmNleftWidget, label);

  entry_name = XmCreateTextField(form, "addEntryNameEdit", args, 2);
  XtManageChild(entry_name);

  /**************************************************
   *
   * Parent name edit field and label.
   * 
   */

  XtSetArg(args[0], XmNtopWidget, entry_name);
  label = XmCreateLabel(form, "addEntryParentNameLabel", args, 1);
  XtManageChild(label);
  
  count = 0;
  if (parentdn && !isnull(*parentdn))
    {
      char buffer[LINESIZE];
      
      QyPrintFriendlyDN(parentdn, buffer);
      XtSetArg(args[count], XmNvalue, buffer); count++;
    }

  XtSetArg(args[count], XmNleftWidget, label); count++;
  XtSetArg(args[count], XmNtopWidget, entry_name); count++;

  parent_name = XmCreateTextField(form, "addEntryParentNameEdit", args, count);
  XtManageChild(parent_name);

  /**************************************************
   *
   * Add entry type label and selection list.
   * 
   */
  
  XtSetArg(args[0], XmNtopWidget, parent_name);
  label = XmCreateLabel(form, "addEntryTypeLabel", args, 1);
  XtManageChild(label);
  
  count = 0;
  XtSetArg(args[count], XmNeditable, FALSE); count++;
  
  if (entry_class_list != NULLEntryClass)
    {
      XtSetArg(args[count], XmNvalue, entry_class_list->name); count++;
    }

  XtSetArg(args[count], XmNleftWidget, label); count++;
  XtSetArg(args[count], XmNtopWidget, parent_name); count++;
  
  type_name = XmCreateTextField(form, "addEntryType", args, count);
  XtManageChild(type_name);
  
  XtSetArg(args[0], XmNleftWidget, type_name);
  XtSetArg(args[1], XmNtopWidget, parent_name);
  
  XtSetArg(args[2], XmNselectionPolicy, XmSINGLE_SELECT);
  XtSetArg(args[3], XmNscrollBarDisplayPolicy, XmSTATIC);

  XtSetArg(args[4], XmNresizable, TRUE);
  list = XmCreateScrolledList(form, "addEntryTypeSelect", args, 5);

  XtManageChild(list);
  XtAddCallback(list,
		XmNsingleSelectionCallback,
		selectClass,
		(XtPointer) NULL);
  
  /***************************************************
   *
   * Get class list and add it to list box.
   *
   */
  
  for (class = entry_class_list, count = 0;
       class != NULLEntryClass;
       class = class->next, count++)
    ;
  
  class_strings = (XmString *) XtMalloc((unsigned) (sizeof(XmString) * count));
  
  for (class = entry_class_list, count = 0;
       class != NULLEntryClass;
       class = class->next, count++)
    class_strings[count] = XmStringCreateSimple(class->name);
  
  XmListAddItems(list, class_strings, (int) count, 1);
  XtAddCallback(list,
		XmNdestroyCallback,
		XLUfreeStorage,
		(XtPointer) class_strings);

  for (strcount = 0; strcount < count; strcount++)
    XmStringFree(class_strings[count]);

  XtFree((XtPointer) class_strings);
  
  /***************************************************
   *
   * Raise the dialog and place it under the pointer.
   *
   */
  
  XtTranslateCoords(toplevel, 0, 0, &x, &y);
  XtRealizeWidget(popup_shell);
  
  if (x < 0)
    x = 1;
  
  if (y < 0)
    y = 1;
  
  count = 0;
  
  XtSetArg(args[count], XmNwidth, &width); count++;
  XtSetArg(args[count], XmNheight, &height); count++;
  
  XtGetValues(form, args, count);
  
  if (WidthOfScreen(XtScreen(toplevel)) < (x + width))
    x = WidthOfScreen(XtScreen(toplevel)) - width - 1;
  
  if (HeightOfScreen(XtScreen(toplevel)) < (x + height))
    y = HeightOfScreen(XtScreen(toplevel)) - height - 1;
  
  XtMoveWidget(popup_shell, x, y);
  XtPopup(popup_shell, XtGrabExclusive);
  
  size_hints.flags = PSize | PPosition | PMinSize;
  
  size_hints.min_width = size_hints.width = (int) width;
  size_hints.min_height = size_hints.height = (int) height;
  
  size_hints.x = (int) x;
  size_hints.y = (int) y;
  
  XStringListToTextProperty(&name, 1, &windowName);
  XStringListToTextProperty(&name, 1, &iconName);
  
  wm_hints.flags = 0;
  
  class_hints.res_class = resource_class;
  class_hints.res_name = program_name;
  
  XSetWMProperties(dpy,
		   XtWindow(popup_shell),
		   &windowName,
		   &iconName,
		   (char **) NULL,
		   0,
		   &size_hints,
		   &wm_hints,
		   &class_hints);
  
  XRaiseWindow(dpy, XtWindow(popup_shell));
} /* addDialog */

/*****************************************************************************
 * addOk()
 *
 * The add dialog has been OK'ed. Check that the entry name has been
 * supplied before returning control.
 *
 ****************************************************************************/

/* ARGSUSED */
static void addOk(w, closure, calldata)
     Widget w;
     XtPointer closure;
     XtPointer calldata;
{
  String entry_name, parent_name, class_name, name_save;
  XLUentryClass class;
  Arg args[MAXARGS];
  int count;
  Widget name_text, class_text, parent_text, form;
  char err_msg[LINESIZE];
  
  form = XtParent(w);
  name_text = XtNameToWidget(form, "addEntryNameEdit");
  parent_text = XtNameToWidget(form, "addEntryParentNameEdit");
  class_text = XtNameToWidget(form, "addEntryType");
  
  count = 0;
  XtSetArg(args[count], XmNvalue, &parent_name); count++;
  XtGetValues(parent_text, args, count);

  /**************************************************
   *
   * First get and check the new RDN.
   * 
   */
  
  if (isemptystr(parent_name))
    {
      (void) sprintf(err_msg, "A parent name must be supplied!");
      XLUpopupMessage(err_msg);
      return;
    }
  
  count = 0;
  XtSetArg(args[count], XmNvalue, &entry_name); count++;
  XtGetValues(name_text, args, count);
  
  if (isemptystr(entry_name))
    {
      (void) sprintf(err_msg, "An entry name must be supplied!", entry_name);
      XLUpopupMessage(err_msg);
      return;
    }

  if (ADDrdn != NULLString)
    XtFree(ADDrdn);

  ADDrdn = XtNewString(entry_name);
  
  /***************************************************
   *
   * Get entry class.
   *
   */
  
  XtSetArg(args[0], XmNvalue, &class_name);
  XtGetValues(class_text, args, 1);
  
  for (class = entry_class_list; class != NULLEntryClass; class = class->next)
    if (lexequ(class->name, class_name) == 0)
      break;
  
  if (class == NULLEntryClass)
    return;
  
  ADDclass = class;
  checkEntryName(parent_name);
  
  XtSetSensitive(form, FALSE);
} /* addOk */

/*****************************************************************************
 * addCancel()
 *
 * The add dialog has been user cancelled.
 *
 ****************************************************************************/

/* ARGSUSED */
static void addCancel(w, closure, calldata)
     Widget w;
     XtPointer closure;
     XtPointer calldata;
{
  Widget shell = XtParent(XtParent(w));
  
  XtPopdown(shell);
  XtDestroyWidget(shell);
  
  XFlush(dpy);
  doPrimitiveSensitivity((int) closure);
} /* addCancel */

/*****************************************************************************
 * selectClass()
 *
 * A class has been selected from the list box. Display it in
 * class test box.
 *
 ****************************************************************************/

/* ARGSUSED */
static void selectClass(w, closure, calldata)
     Widget w;
     XtPointer closure;
     XtPointer calldata;
{
  Widget type;
  Arg args[MAXARGS];
  int count;
  String str;
  
  type = XtNameToWidget(XtParent(XtParent(w)), "addEntryType");
  
  str = XLUgetListString(w, &count);
  if (str == NULLString)
    return;
  
  XtSetArg(args[0], XmNvalue, str);
  XtSetValues(type, args, 1);

  XtFree(str);
} /* selectClass */

/*****************************************************************************
 * checkEntryName()
 *
 * 
 *
 ****************************************************************************/

static void checkEntryName(name, objid)
     String name;
     int objid;
{
  QyErrorCode srchstate;
  int reqid;

  srchstate = QyUFNResolve(NULLEntryList, name, NULLStrCell, &reqid);
  switch (srchstate)
    {
    case QERR_bad_value_syntax:

      XBell(dpy, 40);
      XLUpopupMessage
	("The entry name supplied is invalid!\nCan't continue with addition.");
      
      return;

    case QERR_internal_limit_reached:

      XBell(dpy, 40);
      XLUpopupMessage("Internal limit reached, cannot search!");

      return;
    }

/*  XLUstatusMessage(objid, "Validating parent entry name. Please wait."); */
  XLUrequestInvoked(reqid,
		    UFNAME,
		    gotNameCheckResult,
		    (DSReportCallback) NULL,
		    ADDpw,
		    ADDobjid,
		    ADDprimid,
		    ADDactid,
		    ADDactlist);
} /* checkEntryName */

/*****************************************************************************
 * gotNameCheckResult()
 *
 * 
 *
 ****************************************************************************/

/* ARGSUSED */
static void gotNameCheckResult(reqid, objid, actid, actlist)
     int reqid;
     int objid;
     int actid;
     XLUactionList actlist;
{
  QyUFNResult results;
  QyErrorList errors;
  int err_num, tmplid, childid;
  char strufn[LINESIZE], parent_strufn[LINESIZE], errmsg[LINESIZE];
  XLUentryClass class;
  XLUupdateContext modstate;
  Widget *instances;

  results = QyGetUFNResult(reqid);
  QyRequestComplete(reqid);

  /***************************************************
   *
   * Check the error situation.
   *
   */
  
  for (err_num = 0, errors = results->errors;
       errors != NULLError;
       errors = errors->next, err_num++)
    ;

  XLUsetWindowErrors(objid, results->errors);

  if (err_num > 0)
    updateErrorViewer(objid);

  if (results->type != QyUFNsuccess)
    {
      /***************************************************
       *
       * Couldn't find a context.
       *
       */
      
      XBell(dpy, 40);
      
      (void) strcpy(errmsg, "Couldn't find parent entry!\n");
      (void) strcat(errmsg, "Can't continue with operation.");

      XLUpopupMessage(errmsg);
      XtSetSensitive(XtNameToWidget(add_dialog, "addDialogForm"), TRUE);
      
      QyFreeUFNResult(&results);
      XtFree(ADDrdn);
      ADDrdn = NULLString;
      
      return;
    }

  if (results->matchnum != 1)
    {
      /***************************************************
       *
       * Can't have more than one parent!
       *
       */

      XBell(dpy, 40);

      (void) strcpy(errmsg, "Couldn't match parent name unambiguously!\n");
      (void) strcat(errmsg, "Can't continue with operation.");

      XLUpopupMessage(errmsg);
      XtSetSensitive(XtNameToWidget(add_dialog, "addDialogForm"), TRUE);

      XtFree(ADDrdn);
      ADDrdn = NULLString;
      QyFreeUFNResult(&results);

      return;
    }

  /**************************************************
   *
   * Name checked OK, destroy dialog.
   * 
   */
  
  XtSetSensitive(XtNameToWidget(add_dialog, "addDialogForm"), TRUE);
  XtDestroyWidget(add_dialog);
  add_dialog = NULLWidget;

  /**************************************************
   *
   * Create appropriate output window.
   * 
   */

  (void) XLUgetOutputTemplateID(actlist->outputs, XLUoneHit, &tmplid);
  (void) XLUfindOutputObject(objid, tmplid, actlist->mode, &childid);
  
  displayCurrentEntry(NULLWidget, childid, 0, 0, NULLActionList);
  XLUwait(childid);
  
  /**************************************************
   *
   * Display template for new entry.
   * 
   */

  QyPrintFriendlyDN(results->matches->string_dn, parent_strufn);

  (void) strcpy(strufn, ADDrdn);
  (void) strcat(strufn, ", ");
  (void) strcat(strufn, parent_strufn);

  XtFree(ADDrdn);
  ADDrdn = NULLString;

  instances = XLUgetPrimitiveList(childid, MODIFY);
  if (instances == (Widget *) NULL)
    return;

  displayClassTemplate(strufn, ADDclass, instances[0], childid);

  XLUrequestComplete(reqid);
  QyFreeUFNResult(&results);

  setUpdateState(instances[0], TRUE, Composing);

  /* Set entry class */
  modstate = (XLUupdateContext) XLUgetPrimitiveData(instances[0]);
  modstate->class = ADDclass;

  XtFree((char *) instances);

  /**************************************************
   *
   * Reset sensitivity
   * 
   */

  doPrimitiveSensitivity(childid);
  doPrimitiveSensitivity(objid);
} /* gotNameCheckResult */


