#ifndef lint
static char *rcsid = "$Header: /a/izar/vol/fileservers/izar/izar4/src/local/X500/xd/work.motif/src/RCS/configure.c,v 1.1 94/02/01 16:37:18 ccsrdsm Exp Locker: ccsrdsm $";
#endif

/* $Header: /a/izar/vol/fileservers/izar/izar4/src/local/X500/xd/work.motif/src/RCS/configure.c,v 1.1 94/02/01 16:37:18 ccsrdsm Exp Locker: ccsrdsm $ */

/******************************************************************************

  configure.c - Read screen configure file and build templates

*****************************************************************************/

#include "logger.h"

#include "layout.h"
#include "configure.h"
#include "actions.h"
#include "xdutil.h"
#include "status.h"
#include "defs.h"

#include "ufsearch.h"
#include "dn_list.h"
#include "attrt_list.h"

#include "y.tab.h"
#include "tailor.h"

/***************************************************
 *
 * Externals...
 *
 */

extern XLUwindowTemplate win_templates[];
extern int template_number;

extern int start_template_ids[];
extern int startup_number;

extern XLUobjectAction actions[];
extern int action_number;

#ifndef NO_STATS
extern LLog *log_stat;
#endif

void exit();
extern long strtol();

/* If testing, config files are read from source directory only */
BOOLEAN testing;

/***************************************************
 *
 * Local functions.
 *
 */

static void free_resource_list();
static void check_tailoring();
static void add_resources_to_prim();
static void add_resources_to_layout();
static int add_action_entry();
static void AddOutputDef();

/***************************************************
 *
 * Globals.
 *
 */

char user_path[STRINGLEN],
     config_directory[STRINGLEN],
     config_file_name[LINESIZE],
     config_file_path[LINESIZE];

FILE *configuration_file;
file_org config_origin;

int config_line_num;

/***************************************************
 *
 *     Local data.
 *
 */

static char err_msg[LINESIZE];

/**************************************************
 *
 * Local definitions.
 * 
 */

typedef struct _dn_db
{
  String name;
  String dn;
  struct _dn_db *next;
} dn_db, *DNdb;

#define NULLDNdb (DNdb) NULL
#define dndb_alloc() (DNdb) XtNew(dn_db)

static DNdb entrydb = NULLDNdb;

/**************************************************
 *
 * List attributes by attribute, RDN, etc.
 * 
 */

static XLUlistSelection Selections = NULLListSelection;
static stringCell FullSelection = NULLStrCell;

/***************************************************
 *
 * Paths.
 *
 */

char bitmap_directory[STRINGLEN];
extern char help_directory[];

/*****************************************************************************
 * user_tailor()
 *
 * 
 *
 ****************************************************************************/

void user_tailor()
{
  FILE *config_file;
  char *getenv(), *TidyString(), *SkipSpace();

#ifndef NO_STAT
  ll_hdinit(log_stat, "mxlookup");
#endif

  config_file_path[0] = help_directory[0] = bitmap_directory[0] = '\0';

  /* Use default configuration file name if none specified on cmd line */
  if (*config_file_name == '\0')
    (void) strcpy(config_file_name, "mxlurc");

  (void) strcpy(user_path, getenv("HOME"));

  if (testing != TRUE)
    {
      /* Test run from XLU source tree */
      (void) sprintf(config_directory, "%s%s/", isodetcpath, "Mxlu");
      (void) sprintf(bitmap_directory, "%sbitmaps/", config_directory);
      (void) sprintf(help_directory, "%shelp/", config_directory);
    }
  else
    {
      /* Running an installed XLU, so read config files from right place */
      (void) strcpy(config_directory, "../");
      (void) sprintf(bitmap_directory, "%sbitmaps/", config_directory);
      (void) sprintf(help_directory, "%shelp/", config_directory);
      (void) strcat(config_directory, "config/");
    }

  if (strcmp(config_file_name, "mxlurc") == 0)
    /* Read ~/.mxlurc */
    (void) sprintf(config_file_path, "%s%s", user_path, "/.mxlurc");

  config_file = fopen(config_file_path, "r");
  if (config_file == (FILE *) NULL)
    {
      /* Read named configuration from config directory */
      (void) sprintf(config_file_path,
		     "%s%s",
		     config_directory,
		     config_file_name);

      config_file = fopen(config_file_path, "r");
    }
  
  if (config_file == (FILE *) NULL)
    {
      /* Try reading config file from current directory */
      (void) strcpy(config_file_path, config_file_name);

      config_file = fopen (config_file_path, "r");
      if (config_file == (FILE *) NULL)
	{
	  (void) sprintf(err_msg,
			 "Can't find tailor file %s!",
			 config_file_name);

	  xluFatal(err_msg);
	}
    }
  
  parse_win_config(config_file, SYSTEM_FILE, config_file_path);
} /* user_tailor */

/*****************************************************************************
 * parse_win_config()
 *
 * 
 *
 ****************************************************************************/

void parse_win_config(config_file, file_origin, file_name)
     FILE *config_file;
     file_org file_origin;
     String file_name;
{
  extern int yyparse();

  config_origin = file_origin;
  configuration_file = config_file;

  config_line_num = 1;
  (void) strcpy(config_file_name, file_name);

  (void) yyparse();
  check_tailoring();
} /* parse_win_config */

/*****************************************************************************
 * check_tailoring()
 *
 * 
 *
 ****************************************************************************/

static void check_tailoring()
{
  int count;

  /**************************************************
   *
   * Check that all referenced template
   * names have defined layouts.
   *
   */

  for (count = 0; count < template_number; count++)
    if (win_templates[count]->layout == NULLLayout)
      {
	(void) sprintf(err_msg,
		       "No layout definition for referenced name `%s'.",
		       win_templates[count]->template_name);

	xluFatal(err_msg);
      }
}

/*****************************************************************************
 * add_template_entry()
 *
 * 
 *
 ****************************************************************************/

int add_template_entry(window_name)
     String window_name;
{
  XLUwindowTemplate curr_template;
  int template_index;

  /**************************************************
   *
   * Check to see the given name hasn't already
   * been added to the list.
   * 
   */

  for (template_index = 0; 
       template_index < MAXTEMPLATES
       && template_index < template_number 
       && lexequ(window_name, win_templates[template_index]->template_name);
       template_index++)
    ;

  if (template_index < template_number) return template_index;

  win_templates[template_index] = curr_template = template_alloc();
  win_templates[template_index + 1] = NULLTemplate;

  /**************************************************
   *
   * If unparsed only the names of member
   * templates are stored.
   *
   */

  curr_template->parsed = FALSE;
  curr_template->template_name = XtNewString(window_name);
  curr_template->layout = NULLLayout;
  curr_template->origin = config_origin;

  template_number++;

  return template_index;
} /* add_template_entry */

/*****************************************************************************
 * put_layout()
 *
 * 
 *
 ****************************************************************************/

void put_layout(group_id, new_layout)
     int group_id;
     XLUlayoutObject new_layout;
{
  register XLUlayoutObject curr_layout;
  orientation orient;
  XLUwindowTemplate this_template = win_templates[group_id];

  curr_layout = this_template->layout = new_layout;

  for (orient = curr_layout->orient, curr_layout = curr_layout->next;
       curr_layout != NULLLayout;
       curr_layout = curr_layout->next)
    curr_layout->orient = orient;
} /* put_layout */

/*****************************************************************************
 * add_prim_to_layout()
 *
 * 
 *
 ****************************************************************************/

XLUlayoutObject add_prim_to_layout(primitive_id, resources)
     int primitive_id;
     XLUobjectProperty resources;
{
  XLUlayoutObject prim_member;

  prim_member = layout_alloc();

  prim_member->ref = REF_primitive;
  prim_member->reference.primitive_id = primitive_id;
  prim_member->width = 0;
  prim_member->height = 0;
  prim_member->bitmap_file = NULLString;
  prim_member->help_file = NULLString;
  prim_member->label = NULLString;
  prim_member->name = NULLString;
  prim_member->expand = FALSE;
  prim_member->insensitivity = FALSE;
  prim_member->action_list = NULLPrimActionList;
  prim_member->next = NULLLayout;

  if (resources != NULLObjRes)
    {
      add_resources_to_prim(resources, prim_member);
      free_resource_list(resources);
    }
    
  return prim_member;
} /* add_prim_to_layout */

/*****************************************************************************
 * add_resources_to_prim()
 *
 * 
 *
 ****************************************************************************/

static void add_resources_to_prim(resource_list, prim_member)
     XLUobjectProperty resource_list;
     XLUlayoutObject prim_member;
{
  String action_name;
  XLUprimitiveAction action_list;

  while (resource_list)
    {
    switch (resource_list->type)
      {
      case OT_action:

	action_name = resource_list->resource.action_name;
	action_list = prim_action_alloc();
	action_list->prim_action_id = add_action_entry(action_name);
	action_list->next = prim_member->action_list;
	prim_member->action_list = action_list;
	
	break;
	
      case OT_width:
	prim_member->width = resource_list->resource.width;
	break;
	
      case OT_height:
	prim_member->height = resource_list->resource.height;
	break;
	
      case OT_helpfile:
	prim_member->help_file =
	  XtNewString(resource_list->resource.help_file);
	break;
	
      case OT_bitmap:
	prim_member->bitmap_file =
	  XtNewString(resource_list->resource.bitmap_file);
	break;
	
      case OT_label:
	prim_member->label = XtNewString(resource_list->resource.label);
	break;
	
      case OT_name:
	prim_member->name = XtNewString(resource_list->resource.name);
	break;
	
      case OT_expand:
	prim_member->expand = resource_list->resource.expand;
	break;

      case OT_insensitive:
	prim_member->insensitivity = resource_list->resource.insensitive;
	break;
      }
    
    resource_list = resource_list->next;
  }
} /* add_resources_to_prim */

/*****************************************************************************
 * add_resources_to_layout()
 *
 * 
 *
 ****************************************************************************/

static void add_resources_to_layout(resource_list, this_layout)
     XLUobjectProperty resource_list;
     XLUlayoutObject this_layout;
{
  while (resource_list)
    {
      switch (resource_list->type)
	{
	case OT_bitmap:
	  this_layout->bitmap_file =
	    XtNewString(resource_list->resource.bitmap_file);
	  break;

	case OT_helpfile:
	  this_layout->help_file =
	    XtNewString(resource_list->resource.help_file);
	  break;
	  
	case OT_label:
	  this_layout->label = XtNewString(resource_list->resource.label);
	  break;

	case OT_name:
	  this_layout->name = XtNewString(resource_list->resource.name);
	  break;

	case OT_expand:
	  this_layout->expand = resource_list->resource.expand;
	  break;

	default:
	  break;
	}
      
      resource_list = resource_list->next;
    }
} /* add_resources_to_layout */

/*****************************************************************************
 * add_branch_to_layout()
 *
 * 
 *
 ****************************************************************************/

XLUlayoutObject add_branch_to_layout(template_id, resources)
     int template_id;
     XLUobjectProperty resources;
{
  XLUlayoutObject branch_head = layout_alloc();

  branch_head->ref = REF_layout;
  branch_head->reference.layout_ref_id = template_id;

  branch_head->bitmap_file = NULLString;
  branch_head->label = NULLString;
  branch_head->name = NULLString;
  branch_head->expand = FALSE;
  branch_head->next = NULLLayout;

  if (resources != NULLObjRes)
    {
      add_resources_to_layout(resources, branch_head);
      free_resource_list(resources);
    }
  
  return branch_head;
} /* add_branch_to_layout */

/*****************************************************************************
 * add_to_startup_list()
 *
 * 
 *
 ****************************************************************************/

void add_to_startup_list(template_id)
     int template_id;
{
  start_template_ids[startup_number] = template_id;
  startup_number++;
}

/*****************************************************************************
 * add_action_entry()
 *
 * 
 *
 ****************************************************************************/

static int add_action_entry(action_name)
     String action_name;
{
  int count;
  XLUobjectAction this_action;
  
  if (action_number == MAXACTIONS)
    {
      XLUpopupMessage
	("Maximum number of actions reached! Cannot continue, sorry.\n");

      exit(1);
    }

  for (count = 0;
       count < action_number && 
       lexequ(actions[count]->action_name, action_name);
       count++)
    ;

  if (count == action_number)
    {
      this_action = actions[action_number++] = objaction_alloc();
      this_action->action_name = XtNewString(action_name);
    }
  else
    {
      this_action = actions[count];

      if (this_action->action_name == NULLString)
	this_action->action_name = XtNewString(action_name);
    }

  this_action->to_do = NULLActionList;

  return count;
} /* add_action_entry */

/*****************************************************************************
 * add_action()
 *
 * 
 *
 ****************************************************************************/

void add_action(action_name, actlist)
     String action_name;
     XLUactionList actlist;
{
  XLUobjectAction this_action;
  int action_id;

  action_id = add_action_entry(action_name);
  this_action = actions[action_id];

  this_action->to_do = actlist;
} /* add_action */

/*****************************************************************************
 * put_resource()
 *
 * 
 *
 ****************************************************************************/

XLUobjectProperty put_resource(type, res_value)
     XLUpropertyType type;
     XtPointer res_value;
{
  Dimension dim;
  XLUobjectProperty this_resource = resource_alloc();
  stringCell curr_str;
  char char_val;
  this_resource->type = type;

  switch (type)
    {
    case OT_action:
      this_resource->resource.action_name = XtNewString((String) res_value);
      break;

    case OT_insensitive:
      char_val = 0;

      for (curr_str = (stringCell) res_value;
	   curr_str != NULLStrCell;
	   curr_str = curr_str->next)
	{
	  if (lexequ(curr_str->string, "DSWAIT") == 0)
	    char_val = char_val | I_DS_WAIT;

	  if (lexequ(curr_str->string, "NOERROR") == 0)
	    char_val = char_val | I_NO_ERROR;
	  
	  if (lexequ(curr_str->string, "NOENTRY") == 0)
	    char_val = char_val | I_NO_ENTRY;
	  
	  if (lexequ(curr_str->string, "NOLIST") == 0)
	    char_val = char_val | I_NO_LIST;
	  
	  if (lexequ(curr_str->string, "NORMAL") == 0)
	    char_val = char_val | I_NORMAL;

	  if (lexequ(curr_str->string, "NOMAIL") == 0)
	    char_val = char_val | I_NO_MAIL;
	}

      curr_str = (stringCell) res_value;
      SeqFree(&curr_str);
      
      this_resource->resource.insensitive = char_val;
      break;
      
    case OT_expand:
      this_resource->resource.expand = TRUE;
      break;
      
    case OT_width:
      dim = (Dimension) strtol((char *) res_value, (char **) NULL, 10);
      this_resource->resource.width = dim;
      break;
      
    case OT_height:
      dim = (Dimension) strtol((char *) res_value, (char **) NULL, 10);
      this_resource->resource.height = dim;
      break;
      
    case OT_bitmap:
      this_resource->resource.bitmap_file = XtNewString((String) res_value);
      break;

    case OT_helpfile:
      this_resource->resource.help_file = XtNewString((String) res_value);
      break;
      
    case OT_label:
      this_resource->resource.label = XtNewString((String) res_value);
      break;

    case OT_name:
      this_resource->resource.name = XtNewString((String) res_value);
      break;
    }

  this_resource->next = NULLObjRes;
  return this_resource;
} /* put_resource */

/*****************************************************************************
 * free_resource_list()
 *
 * 
 *
 ****************************************************************************/

static void free_resource_list(resource_list)
     XLUobjectProperty resource_list;
{
  XLUobjectProperty next_resource;

  while (resource_list != NULLObjRes)
    {
      next_resource = resource_list->next;

      switch (resource_list->type)
	{
	case OT_action:
	  XtFree((char *) resource_list->resource.action_name);
	  
	  break;
	  
	case OT_bitmap:
	  XtFree((char *) resource_list->resource.bitmap_file);
	  
	  break;

	case OT_helpfile:
	  XtFree((char *) resource_list->resource.help_file);
	  
	  break;
	  
	case OT_label:
	  XtFree((char *) resource_list->resource.label);

	  break;
	  
	case OT_name:
	  XtFree((char *) resource_list->resource.name);
	  
	  break;

	default:
	  break;
      }

      XtFree((char *) resource_list);
      resource_list = next_resource;
    }
}

/*****************************************************************************
 * new_actlist()
 *
 * 
 *
 ****************************************************************************/

XLUactionList new_actlist(type, name, mode, param, outputs)
     XLUactionType type;
     String name;
     XLUwindowMode mode;
     XtPointer param;
     stringList outputs;
{
  XLUactionList actlist;
  
  extern int yyerror();

  /* Do some simple checking. */
  switch (type)
    {
    case AT_Activate:

      if (SeqCount(outputs) != 3)
	yyerror((char *) NULL);

      break;

    case AT_RaiseObject:

      if (SeqCount(outputs) != 1)
	yyerror((char *) NULL);

      break;
    }

  /* Add new action element. */
  actlist = actlist_alloc();
  actlist->type = type;
  actlist->outputs = (XLUoutputDef) NULL;

  /* Set output template IDs. */
  switch (type)
    {
    case AT_Activate:

      if (*outputs->string != '\0')
	AddOutputDef(&actlist->outputs, XLUmanyHits, outputs->string);
      outputs = outputs->next;
  
      if (*outputs->string != '\0')
	AddOutputDef(&actlist->outputs, XLUoneHit, outputs->string);
      outputs = outputs->next;
      
      if (*outputs->string != '\0')
	AddOutputDef(&actlist->outputs, XLUlogTo, outputs->string);

      break;

    case AT_RaiseObject:
      
      AddOutputDef(&actlist->outputs, XLUraise, outputs->string);
      break;
    }
      
  /* Only used for primitive activations */
  if (name != NULLString)
    actlist->name = XtNewString(name);
  else
    actlist->name = NULLString;

  actlist->mode = mode;
  actlist->param = (char *) param;

  actlist->next = NULLActionList;
  return actlist;
} /* new_actlist */

/*****************************************************************************
 * AddOutputDef()
 *
 * Add to the head of a list of XLUoutputDef structures.
 *
 ****************************************************************************/

static void AddOutputDef(opptr, type, tmplname)
     XLUoutputDef *opptr;
     XLUoutputType type;
     String tmplname;
{
  XLUoutputDef newdef;

  newdef = XtNew(struct XLUoutputDef);
  newdef->type = type;
  newdef->outputTmplID = add_template_entry(tmplname);
  newdef->next = *opptr;
  *opptr = newdef;
} /* AddOutputDef */

/*****************************************************************************
 * set_entry_group()
 *
 * 
 *
 ****************************************************************************/

void set_entry_group(name, description, attributes)
     String name;
     String description;
     stringCell attributes;
{
  extern XLUattributeGroup attr_group_list;
  XLUattributeGroup newgrp;
  attrTypeList type_list;
  stringCell attr_name;
  AttributeType new_type;

  newgrp = attr_group_alloc();
  if (name == NULLString)
    return;

  newgrp->name = XtNewString(name);

  newgrp->description =
    (description == NULLString? NULLString : XtNewString(description));

  for (type_list = NULLAttrTypeList, attr_name = attributes;
       attr_name != NULLStrCell;
       attr_name = attr_name->next)
    {
      new_type = AttrT_new(attr_name->string);
      if (new_type == NULLAttrT)
	{
	  (void) sprintf(err_msg,
			 "Invalid attribute type `%s' in file %s\n",
			 attr_name->string,
			 config_file_name);

	  xluWarning(err_msg);
	  continue;
	}

      QyATListAdd(new_type, &type_list);
    }
  
  newgrp->attributes = type_list;
  newgrp->next = attr_group_list;

  attr_group_list = newgrp;
} /* set_entry_group */

/*****************************************************************************
 * set_entry_class()
 *
 * 
 *
 ****************************************************************************/

void set_entry_class(type_name,
		     key_oc_name,
		     rdn_type,
		     str_oc_list, 
		     str_group_list)
     String type_name;
     String key_oc_name;
     String rdn_type;
     stringCell str_oc_list;
     stringCell str_group_list;
{
  extern XLUentryClass entry_class_list;
  XLUentryClass curr_class;
  AttributeType oc_type;
  attrTypeList oc_list = NULLAttrTypeList;
  stringCell curr_str_oc;
  
  /***************************************************
   *
   * Do some checking.
   *
   */

  if (isemptystr(type_name))
    return;

  for (curr_class = entry_class_list;
       curr_class != NULLEntryClass;
       curr_class = curr_class->next)
    if (!lexequ(curr_class->name, type_name))
      return;

  if (key_oc_name == NULLString)
    return;

  /***************************************************
   *
   * Create a new structure and add
   * to global list.
   *
   */
  
  curr_class = entry_class_alloc();

  curr_class->next = entry_class_list;
  curr_class->name = XtNewString(type_name);

  for (curr_str_oc = str_oc_list;
       curr_str_oc != NULLStrCell;
       curr_str_oc = curr_str_oc->next)
    {
      oc_type = AttrT_new(curr_str_oc->string);
      if (oc_type == NULLAttrT)
	{
	  (void) sprintf(err_msg,
		   "Object class `%s' declared before line %d unrecognized!\n",
			 curr_str_oc->string,
			 config_line_num);
	  
	  xluFatal(err_msg);	  
	}
      else
	(void) QyATListAdd(oc_type, &oc_list);
    }

  oc_type = AttrT_new(key_oc_name);
  if (oc_type == NULLAttrT)
    {
      (void) sprintf(err_msg,
		    "ObjectClass `%s' declared before line %d unrecognized!\n",
		     key_oc_name,
		     config_line_num);

      xluFatal(err_msg);
    }

  curr_class->rdn_type = AttrT_new(rdn_type);
  if (curr_class->rdn_type == NULLAttrT)
    {
      (void) sprintf(err_msg,
		     "Attribute `%s' declared before line %d unrecognized!\n",
		     rdn_type,
		     config_line_num);

      xluFatal(err_msg);
    }

  (void) QyATListAdd(oc_type, &oc_list);

  curr_class->oc_list = oc_list;
  curr_class->groups = str_group_list;

  entry_class_list = curr_class;
} /* set_entry_class */

/*****************************************************************************
 * get_class_of_name()
 *
 * 
 *
 ****************************************************************************/

XLUentryClass get_class_of_name(name)
     String name;
{
  register XLUentryClass curr_class;
  extern XLUentryClass entry_class_list;

  if (name == NULLCP || *name == '\0')
    return NULLEntryClass;

  for (curr_class = entry_class_list;
       curr_class != NULLEntryClass && lexequ(curr_class->name, name) != 0;
       curr_class = curr_class->next)
    ;

  return curr_class;
} /* get_class_of_name */

/*****************************************************************************
 * set_base_type()
 *
 * 
 *
 ****************************************************************************/

void set_base_class(class_name)
     String class_name;
{
  extern String base_entry_class;

  base_entry_class = XtNewString(class_name);
} /* set_base_class */

/*****************************************************************************
 * set_attr_template()
 *
 * 
 *
 ****************************************************************************/

void set_attr_template(type_name, class_name_list, str_attr_list)
     String type_name;
     stringCell class_name_list;
     stringCell str_attr_list;
{
  extern XLUattrTemplate attr_template_list;
  XLUattrTemplate this_tmpl;
  AttributeType this_attr;

  this_attr = AttrT_new(type_name);
  if (this_attr == NULLAttrT)
    {
      (void) sprintf(err_msg,
		     "Attribute `%s' declared at line %d unrecognized!\n",
		     type_name,
		     config_line_num);

      SeqFree(&class_name_list);
      SeqFree(&str_attr_list);
      
      xluWarning(err_msg);
      return;
    }

  this_tmpl = attr_tmpl_alloc();

  this_tmpl->next = attr_template_list;
  attr_template_list = this_tmpl;

  this_tmpl->defaults = str_attr_list;
  this_tmpl->class_names = class_name_list;

  this_tmpl->type = this_attr;
} /* set_attr_template */

/*****************************************************************************
 * add_display_attr()
 *
 * 
 *
 ****************************************************************************/

void add_display_attr(attr_name, showtype)
     String attr_name;
     XLUshowType showtype;
{
  extern XLUreadAttribute attrs_to_display;
  XLUreadAttribute new;
  AttributeType type;

  if (attr_name == NULLString || *attr_name == '\0')
    return;

  if ((type = AttrT_new(attr_name)) == NULLAttrT)
    return;


  new = read_attr_alloc();
  new->next = attrs_to_display;

  new->type = type;
  new->showtype = showtype;

  attrs_to_display = new;
}

/*****************************************************************************
 * add_attribute_name()
 *
 * 
 *
 ****************************************************************************/

void add_attribute_name(attr_name, user_name)
     String attr_name;
     String user_name;
{
  AttributeType type = AttrT_new(attr_name);
  XLUattributeName new_name;
  extern XLUattributeName attr_name_list;

  if (type == NULLAttrT)
    return;

  if (user_name == NULLString || *user_name == '\0')
    return;

  new_name = attr_name_alloc();
  new_name->next = attr_name_list;

  new_name->type = type;
  new_name->name = XtNewString(user_name);

  attr_name_list = new_name;
} /* add_attribute_name */

/*****************************************************************************
 * add_ufs_search_class()
 *
 * 
 *
 ****************************************************************************/

void add_ufs_search_class(class_name,
			  str_oc_type,
			  filter,
			  str_oc_list)
     String class_name;
     String str_oc_type;
     QyFilter filter;
     stringCell str_oc_list;
{
  attrTypeList base_oc_list = NULLAttrTypeList;
  AttributeType target_oc_type, base_oc_type;
  stringCell curr_str_type;
  char msg[LINESIZE];
  
  target_oc_type = AttrT_new(name2oc(str_oc_type)->oc_ot.ot_stroid);
  if (target_oc_type == NULLAttrT)
    {
      (void) sprintf(msg,
   "\"%s\" - Fatal error at or before line %d \n: Invalid object class `%s'.",
		    config_file_path,
		    config_line_num,
		    str_oc_type);

      XLUpopupMessage(msg);
      exit(1);
    }

  for (curr_str_type = str_oc_list;
       curr_str_type != NULLStrCell;
       curr_str_type = curr_str_type->next)
    {
      if (*curr_str_type->string == '\0')
	{
	  QyATListAdd(NULLAttrT, &base_oc_list);
	  continue;
	}

      base_oc_type= AttrT_new(name2oc(curr_str_type->string)->oc_ot.ot_stroid);
      if (base_oc_type == NULLAttrT)
	{
	  (void) sprintf(msg,
  "\"%s\" - Fatal error at or before line %d: Unrecognized object class `%s'.",
			 config_file_path,
			 config_line_num,
			 curr_str_type->string);

	  XLUpopupMessage(msg);
	  exit(1);
	}

      QyATListAdd(base_oc_type, &base_oc_list);
    }
  
  if (QyAddSearchClass(class_name, target_oc_type, filter, base_oc_list)
      == FALSE)
    {
      (void) sprintf(msg,
		     "\"%s\" - Fatal error at or before line %d: Maximum number of search classes exceeded",
		      config_file_path,
		      config_line_num);
	  
      XLUpopupMessage(msg);
      exit (1);
    }
} /* add_ufs_search_class */

/*****************************************************************************
 * add_dn_db()
 *
 * 
 *
 ****************************************************************************/

void add_dn_db(name, dn)
     String name;
     String dn;
{
  DNdb add;

  add = dndb_alloc();
  add->next = entrydb;
  entrydb = add;

  add->name = XtNewString(name);
  add->dn = XtNewString(dn);
} /* add_dn_db */

/*****************************************************************************
 * get_dn_db()
 *
 * 
 *
 ****************************************************************************/

String get_dn_db(name)
     String name;
{
  DNdb curr;

  for (curr = entrydb; curr != NULLDNdb; curr = curr->next)
    {
      if (lexequ(curr->name, name) == 0)
	return curr->dn;
    }

  return name;
} /* get_dn_db */

/*****************************************************************************
 * AddListSelection()
 *
 * 
 *
 ****************************************************************************/

void AddListSelection(oclasses, showdn, showsel, getsel)
     stringCell oclasses;
     BOOLEAN showdn;
     XLUlistValue showsel;
     stringCell getsel;
{
  XLUlistSelection sel;
  XLUlistValue listval;
  QySearchClass class;
  stringCell currstr, copystr;

  if (oclasses == NULLStrCell)
    return;

  sel = (XLUlistSelection) malloc(sizeof(struct XLUlistSelection));
  sel->next = Selections;
  sel->showdn = showdn;

  Selections = sel;

  sel->oclasses = oclasses;
  sel->listvals = showsel;

  /* Add displayed selection to complete list of attribute selections */
  for (listval = showsel;
       listval != (XLUlistValue) NULL;
       listval = listval->next)
    {
      copystr = SeqCopy(listval->attrs);
      for (currstr = copystr;
	   currstr->next != NULLStrCell;
	   currstr = currstr->next)
	;

      currstr->next = FullSelection;
      FullSelection = copystr;
    }

  /* Add hidden selection to complete list of attribute selections */
  if (getsel != NULLStrCell)
    {
      for (currstr = getsel;
	   currstr->next != NULLStrCell;
	   currstr = currstr->next)
	;

      currstr->next = FullSelection;
      FullSelection = getsel;
    }
} /* AddListSelection */

/*****************************************************************************
 * GetFullSelection()
 *
 * 
 *
 ****************************************************************************/

stringCell GetFullSelection()
{
  return FullSelection;
} /* GetFullSelection */

/*****************************************************************************
 * GetListSelection()
 *
 * 
 *
 ****************************************************************************/

XLUlistSelection GetListSelection(oclist)
     stringCell oclist;
{
  XLUlistSelection sel;
  register stringCell stroc;
  stringCell ocvals, currov;
  BOOLEAN got_it = FALSE;
  AttributeType octype;
  objectclass *oc;

  extern stringCell MakeValueSequence();

  ocvals = MakeValueSequence(oclist);
  for (currov = ocvals; currov != NULLStrCell; currov = currov->next)
    {
      oc = name2oc(currov->string);
      if (oc == (objectclass *) NULL)
	continue;
      
      octype = str2AttrT(oc->oc_ot.ot_stroid);
      for (sel = Selections; sel != (XLUlistSelection) NULL; sel = sel->next)
	{
	  for (stroc = sel->oclasses;
	       stroc != NULLStrCell;
	       stroc = stroc->next)
	    {
	      if (octype ==
		  str2AttrT(name2oc(stroc->string)->oc_ot.ot_stroid))
		{
		  got_it = TRUE;
		  break;
		}
	    }
	  
	  if (got_it == TRUE)
	    break;
	}
      
      if (got_it == TRUE)
	break;
    }
  
  SeqFree(&ocvals);

  if (got_it == TRUE)
    return sel;

  return (XLUlistSelection) NULL;
} /* GetListSelection */







