#ifndef lint
static char *rcsid = "$Header: /a/izar/vol/fileservers/izar/izar4/src/local/X500/xd/work.motif/src/RCS/errview.c,v 1.1 94/02/01 16:37:23 ccsrdsm Exp Locker: ccsrdsm $";
#endif

/* $Header: /a/izar/vol/fileservers/izar/izar4/src/local/X500/xd/work.motif/src/RCS/errview.c,v 1.1 94/02/01 16:37:23 ccsrdsm Exp Locker: ccsrdsm $ */

/*****************************************************************************

  errview.c - View errors dialog box
  
*****************************************************************************/

#include "defs.h"

#include <X11/Xatom.h>
#include <Xm/Form.h>
#include <Xm/Label.h>
#include <Xm/PushB.h>
#include <Xm/ScrolledW.h>
#include <Xm/Text.h>

#include "query.h"
#include "quipu/common.h"

#include "windows.h"
#include "objects.h"
#include "callbacks.h"
#include "configure.h"
#include "actions.h"
#include "y.tab.h"
#include "layout.h"
#include "xdutil.h"
#include "xutil.h"

#include "errview.h"

/***************************************************
 *
 * Externals..
 *
 */

extern Display *dpy;

/***************************************************
 *
 * Local functions..
 *
 */

static void seeDetailedError();
static void createErrorField();

/*****************************************************************************
 * updateErrorViewer()
 *
 * Re-display error messages.
 *
 ****************************************************************************/

void updateErrorViewer(object_id)
     int object_id;
{
  WidgetList prim_insts;
  Widget view_form, label, curr_text, prev_button = NULLWidget, curr_button;
  XLUwindowObject this_object;
  char err_msg[STRINGLEN], fname[STRINGLEN], strbuf[STRINGLEN];
  QyErrorList curr_error;
  int error_num;
  Arg args[MAXARGS];
  XtCallbackList callbacks;
  errorField curr_field, next_field, prev_field, first_field = NULLErrorField;
  XmString cstr;

  this_object = XLUgetWindowObject(object_id);

  /***************************************************
   *
   * Find instance of errorViewer in this object.
   *
   */

  prim_insts = XLUgetPrimitiveList(object_id, ERROR_VIEW);
  if (prim_insts == (Widget *) NULL)
    return;

  /***************************************************
   *
   * Count number of errors to display.
   *
   */

  for (curr_error = this_object->current_errors, error_num = 0;
       curr_error != NULLError;
       curr_error = curr_error->next, error_num++)
    ;

  XtSetArg(args[0], XmNworkWindow, &view_form);
  XtGetValues(prim_insts[0], args, 1);

  curr_button = XtNameToWidget(view_form, "errorButton");
  label = XtNameToWidget(view_form, "errorViewLabel");

  (void) sprintf(err_msg, "%d errors", error_num);

  cstr = XmStringCreateSimple(err_msg);
  XtSetArg(args[0], XmNlabelString, cstr);

  XtSetValues(label, args, 1);
  XmStringFree(cstr);

  if (curr_button != NULLWidget)
    {
      XtSetArg(args[0], XmNtopWidget, &prev_button);
      XtGetValues(curr_button, args, 1);
      
      while (prev_button != label)
	{
	  curr_button = prev_button;
	  XtGetValues(curr_button, args, 1);
	}
       
      XtSetArg(args[0], XmNactivateCallback, &callbacks);
      XtGetValues(curr_button, args, 1);
      
      for (curr_field = (errorField) callbacks[0].closure,
	   curr_error = this_object->current_errors;
	   curr_error != NULLError && curr_field != NULLErrorField;
	   curr_error = curr_error->next, curr_field = curr_field->next)
	{
	  if (first_field != NULLErrorField)
	    first_field = curr_field;

	  curr_text = curr_field->text;
	  curr_field->error = curr_error;

	  if (curr_field->error->baseobject != NULLCP)
	    QyPrintFriendlyDN(curr_field->error->baseobject, (String) fname);
	  else
	    fname[0] = '\0';

	  /* Get basic error message. */
	  (void) strcpy
	    (err_msg, QyPrintErrcode(curr_field->error->error_type));

	  if (fname[0] != '\0')
	    (void) sprintf(strbuf, "%s\n Occured at (\"%s\")", err_msg, fname);

	  XtSetArg(args[0], XmNvalue, strbuf);
	  XtSetArg(args[1], XmNtopCharacter, 0);
	  XtSetArg(args[2], XmNcursorPosition, 0);

	  XtSetValues(curr_text, args, 3);
	  XmTextShowPosition(curr_text, 0);
	}

      /***************************************************
       *
       * Have replaced the error text in as many
       * existing fields as possible. If don't have
       * enough fields make some more.
       *
       */

      if (curr_field == NULLErrorField)
	{
	  for (curr_field = (errorField) callbacks[0].closure;
	       curr_field->next != NULLErrorField;
	       curr_field = curr_field->next)
	    ;

	  /***************************************************
	   *
	   * Create fields for each remaining error.
	   *
	   */

	  while (curr_error != NULLError)
	    {
	      next_field = curr_field->next = error_field_alloc();

	      if (first_field != NULLErrorField)
		first_field = next_field;

	      next_field->next = NULLErrorField;
	      next_field->error = curr_error;

	      createErrorField(next_field, curr_field->text, view_form);
	    }
	}
      else if (curr_field != NULLErrorField &&
	       curr_field != (errorField) callbacks[0].closure)
	/***************************************************
	 *
	 * Have displayed all required errors. If have
	 * text fields remaining from the last set of
	 * errors, destroy them.
	 *
	 */
	{
	  next_field = curr_field;

	  for (curr_field = (errorField) callbacks[0].closure;
	       curr_field != next_field && curr_field->next != next_field;
	       curr_field = curr_field->next)
	    ;

	  curr_field->next = NULLErrorField;

	  for (curr_field = next_field, next_field = curr_field->next;
	       curr_field != NULLErrorField;
	       curr_field = next_field, next_field = curr_field->next)
	    {
	      curr_field->error->next = NULLError;
	      QyFreeErrorList(&curr_field->error);
	      
	      XtSetArg(args[0], XmNleftWidget, &curr_button);
	      XtGetValues(curr_button, args, 1);

	      XtUnmanageChild(curr_field->text);
	      XtUnmanageChild(curr_button);

	      XtDestroyWidget(curr_field->text);
	      XtDestroyWidget(curr_button);
	      
	      XtFree((XtPointer) curr_field);
	    }
	}
    }
  else
    /***************************************************
     *
     * Need to create a button/text field
     * pair for each error.
     *
     */
    {
      for (curr_error = this_object->current_errors,
	   curr_field = prev_field = NULLErrorField;
	   curr_error != NULLError;
	   prev_field = curr_field,
	   curr_field = NULLErrorField,
	   curr_error = curr_error->next)
	{
	  if (prev_field == NULLErrorField)
	    first_field = curr_field = error_field_alloc();
	  else
	    curr_field = prev_field->next = error_field_alloc();
	  
	  curr_field->next = NULLErrorField;
	  curr_field->error = curr_error;

	  createErrorField
	    (curr_field,
	     (prev_field == NULLErrorField? label : prev_field->text),
	     view_form);

	  XFlush(dpy);

	  if (prev_field != NULLErrorField)
	    XmTextShowPosition(prev_field->text, 0);
	}
    }

  while (first_field != NULLErrorField)
    {
      XmTextShowPosition(first_field->text, 0);
      first_field = first_field->next;
    }
} /* updateErrorViewer */

/*****************************************************************************
 * seeDetailedError()
 *
 * Display a more system oriented error message.
 *
 ****************************************************************************/

/* ARGSUSED */
static void seeDetailedError(w, closure, calldata)
     Widget w;
     XtPointer closure;
     XtPointer calldata;
{
  Dimension width, height;
  Arg args[MAXARGS];
  int count;
  errorField this_field = (errorField) closure;

  XtSetArg(args[0], XmNvalue, this_field->error->ds_message);
  XtSetValues(this_field->text, args, 1);

  XLUgetTextExtents(this_field->text,
		    this_field->error->ds_message,
		    &width,
		    &height);

  count = 0;
  
  XtSetArg(args[count], XmNwidth, (Dimension) width); count++;
  XtSetArg(args[count], XmNheight, (Dimension) height); count++;

  XtSetValues(this_field->text, args, count);
} /* seeDetailedError */

/*****************************************************************************
 * createErrorField()
 *
 * 
 *
 ****************************************************************************/

static void createErrorField(this_field, from_vert, parent)
     errorField this_field;
     Widget from_vert;
     Widget parent;
{
  int count;
  Arg args[MAXARGS];
  Widget curr_button;
  char fname[STRINGLEN], err_msg[STRINGLEN], strbuf[STRINGLEN];

  /* Get baseobject of erroneous request. */
  if (this_field->error->baseobject != NULLCP)
    QyPrintFriendlyDN(this_field->error->baseobject, (String) fname);
  else
    fname[0] = '\0';

  /* Get basic error message. */
  (void) strcpy(err_msg, QyPrintErrcode(this_field->error->error_type));

  if (fname[0] != '\0')
    {
      (void) sprintf(strbuf, "\n Occured at (\"%s\")", fname);
      (void) strcat(err_msg, strbuf);
    }

  /* Create Examine error button. */

  count = 0;
  if (from_vert != NULLWidget)
    {
      XtSetArg(args[count], XmNtopAttachment, XmATTACH_WIDGET); count++;
      XtSetArg(args[count], XmNtopWidget, from_vert); count++;
    }
  else
    {
      XtSetArg(args[count], XmNtopAttachment, XmATTACH_FORM); count++;
    }

  curr_button = XmCreatePushButton(parent, "errorButton", args, count);
  XtManageChild(curr_button);

  /* Callback for viewing detailed errors. */
  XtAddCallback(curr_button,
		XmNactivateCallback,
		seeDetailedError,
		(XtPointer) this_field);

  /* Insensitive if no detailed error message. */
  if (this_field->error->ds_message == NULLCP)
    XtSetSensitive(curr_button, FALSE);
  else
    XmAddTabGroup(curr_button);

  /* Create error text field. */
  count = 0;

  XtSetArg(args[count], XmNeditable, FALSE); count++;
  XtSetArg(args[count], XmNtopCharacter, 0); count++;

  XtSetArg(args[count], XmNresizeWidth, TRUE); count++;
  XtSetArg(args[count], XmNresizeHeight, TRUE); count++;

  XtSetArg(args[count], XmNresizable, TRUE); count++;
  
  XtSetArg(args[count], XmNleftWidget, curr_button); count++;
  XtSetArg(args[count], XmNcursorPositionVisible, TRUE); count++;

  if (from_vert != NULLWidget)
    {
      XtSetArg(args[count], XmNtopAttachment, XmATTACH_WIDGET); count++;
      XtSetArg(args[count], XmNtopWidget, from_vert); count++;
    }
  else
    {
      XtSetArg(args[count], XmNtopAttachment, XmATTACH_FORM); count++;
    }
  
  this_field->text = XmCreateText(parent, "errorText", args, count);

  XtSetArg(args[0], XmNvalue, (String) err_msg); count++;
  XtSetValues(this_field->text, args, 1);

  XtSetArg(args[0], XmNtopCharacter, 0);
  XtSetArg(args[1], XmNcursorPosition, 0);
  XtSetValues(this_field->text, args, 2);

  XmTextShowPosition(this_field->text, 0);

  XtManageChild(this_field->text);
  XtRealizeWidget(this_field->text);
  XmAddTabGroup(this_field->text);
} /* createErrorField */



