#ifndef lint
static char *rcsid = "$Header: /a/izar/vol/fileservers/izar/izar4/src/local/X500/xd/work.motif/src/RCS/layout.c,v 1.1 94/02/01 16:37:44 ccsrdsm Exp Locker: ccsrdsm $";
#endif

/* $Header: /a/izar/vol/fileservers/izar/izar4/src/local/X500/xd/work.motif/src/RCS/layout.c,v 1.1 94/02/01 16:37:44 ccsrdsm Exp Locker: ccsrdsm $ */

/*****************************************************************************

  layout.c - Prcoedures to produce and layout groups of widgets.

*****************************************************************************/

#include "defs.h"
#include "windows.h"
#include "widgets.h"
#include "layout.h"
#include "xutil.h"
#include "objects.h"
#include "actions.h"
#include "configure.h"
#include "y.tab.h"
#include "help.h"

#include <X11/cursorfont.h>
#include <X11/Xutil.h>
#include <X11/Shell.h>
#include <Xm/ScrolledW.h>
#include <Xm/PushB.h>
#include <Xm/Text.h>
#include <Xm/Label.h>
#include <Xm/Form.h>
#include <Xm/MenuShell.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>

/***************************************************
 *
 * Externals.
 *
 */

extern Display *dpy;

extern int template_number;
extern XLUwindowTemplate win_templates[];
extern Widget toplevel;

/***************************************************
 *
 * Local functions.
 *
 */

static void XLUmakeWindowFromTemplateAux();
static Widget makePrimitive();
static void doLayout(), layoutPrimitive();
static XLUlayoutObject XLUgetPrimitiveTemplateAux();
static void XLUgetPrimitiveListAux(), findNamedInstancesAux();

/***************************************************
 *
 * Local data.
 *
 */

Widget widget_list[MAXMEMBERS];
int widget_count;

/*****************************************************************************
 * XLUmakeWindowFromTemplate()
 *
 * Make a group of widgets as defined by the given layout
 * tree. If no parent given make one and return a pointer
 * to it.
 * 
 ****************************************************************************/

Widget XLUmakeWindowFromTemplate(template_id, objid, parent)
     int template_id;
     int objid;
     Widget parent;
{
  Arg args[MAXARGS];
  Widget form;
  XLUwindowTemplate template = win_templates[template_id];
  XLUwindowObject this_object;
  WidgetTree top_layout;
  XSizeHints szhints;
  long hintflags;
  Window parentwin;

  this_object = XLUgetWindowObject(objid);

  /***************************************************
   *
   * If no parent given, create a new parent
   * (= topLevelShell).
   *
   */

  if (parent == NULLWidget)
    {
      parent = XtCreatePopupShell(template->template_name, 
				  topLevelShellWidgetClass,
				  toplevel,
				  args,
				  0);
    }
    

  /***************************************************
   *
   * Create head of widget map of this object.
   *
   */

  top_layout = WidgetTree_alloc();
  top_layout->left_siblings = NULLWidgetTree;
  top_layout->reference = REF_layout;
  top_layout->descendants = NULLWidgetTree;

  /***************************************************
   *
   * Call recursive create procedure.
   *
   */

  XLUmakeWindowFromTemplateAux(objid,
			       template_id,
			       template->layout, 
			       &top_layout->descendants,
			       parent, 
			       &form,
			       NULLWidget,
			       NULLWidget,
			       FALSE,
			       FALSE,
			       horizontal);


  /***************************************************
   *
   * Put parent pointer.
   *
   */

  top_layout->this_widget = form;

  /***************************************************
   *
   * Finalise description of this object,
   * i.e. point to widget map and parent shell.
   *
   */

  this_object->widget_layout = top_layout;
  this_object->parent_shell = parent;

  XtRealizeWidget(parent);

  /**************************************************
   *
   * Set window manager hints.
   * 
   */

  parentwin = XtWindow(parent);
  XGetWMNormalHints(dpy, parentwin, &szhints, &hintflags);

  if (parent == toplevel)
    /* Use command line for position (not size or gravity) of first window. */
    szhints.flags = USPosition | PSize | PWinGravity;
  else
    {
      /* Set program defined start position (can be ignored by WM). */
      XLUplaceWindow(szhints.width, szhints.height, &szhints.x, &szhints.y);
      szhints.flags = USPosition | PSize | PWinGravity;
    }

  XSetWMNormalHints(dpy, parentwin, &szhints);
  XtMoveWidget(parent, szhints.x, szhints.y);

  /**************************************************
   *
   * Make sure that window manager events
   * are processed properly.
   * 
   */

  XLUsetWMHandler(parent);

  /***************************************************
   *
   *	Force required layout changes to ensure 
   * 	all available space is taken up.
   *
   */

  doLayout(template->layout, top_layout->descendants, form);

  /***************************************************
   *
   * Initialize action contexts and
   * other data contexts.
   *
   */

  XLUsaveContextData(template->layout, top_layout->descendants, objid);

  /***************************************************
   *
   * Return pointer to parent shell.
   *
   */

  return parent;
} /* XLUmakeWindowFromTemplate */

/*****************************************************************************
 * XLUmakeWindowFromTemplateAux()
 *
 * Recursive procedure to build widgets and a map of widgets from
 * a given layout.
 *
 ****************************************************************************/

static void XLUmakeWindowFromTemplateAux(objid,
					 tmplid,
				template,
				widget_layout_ptr,
				parent,
				form,
				from_vert,
				from_horiz,
				expand_this,
				follow_expand,
				parent_orient)
     int objid;
     int tmplid;
     XLUlayoutObject template;
     WidgetTree *widget_layout_ptr;
     Widget parent;
     Widget *form;
     Widget from_vert;
     Widget from_horiz;
     BOOLEAN expand_this;
     BOOLEAN follow_expand;
     orientation parent_orient;
{
  XLUlayoutObject curr_layout = template;
  orientation curr_orientation = curr_layout->orient;
  WidgetTree *curr_sibling, *curr_descendant;
  Widget child_form, curr_pw;
  Arg args[MAXARGS];
  int count;
  BOOLEAN had_child_expand = FALSE;

  /* Create the parent formWidget. */
  /* If this form is placed relative to another, set separation distance */

  count = 0;
  XtSetArg(args[count], XmNinitialResourcesPersistent, FALSE); count++;

  if (XtClass(parent) == xmFormWidgetClass)
    {
      if (parent_orient == horizontal)
	{
	  XtSetArg(args[count], XmNtopAttachment, XmATTACH_FORM); count++;
	  XtSetArg(args[count], XmNbottomAttachment, XmATTACH_FORM); count++;
	  
	  if (from_horiz != NULLWidget && had_child_expand == FALSE)
	    {
	      XtSetArg(args[count], XmNleftAttachment, XmATTACH_WIDGET);
	      	count++;
	      XtSetArg(args[count], XmNleftWidget, from_horiz); count++;
	    }
	  
	  if (from_horiz == NULLWidget)
	    {
	      XtSetArg(args[count], XmNleftAttachment, XmATTACH_FORM);count++;
	    }
	}
      else
	{
	  XtSetArg(args[count], XmNleftAttachment, XmATTACH_FORM); count++;
	  XtSetArg(args[count], XmNrightAttachment, XmATTACH_FORM); count++;

	  if (from_vert != NULLWidget && had_child_expand == FALSE)
	    {
	      XtSetArg(args[count], XmNtopAttachment, XmATTACH_WIDGET);
	      	count++;
	      XtSetArg(args[count], XmNtopWidget, from_vert); count++;
	    }
	  
	  if (from_vert == NULLWidget)
	    {
	      XtSetArg(args[count], XmNtopAttachment, XmATTACH_FORM); count++;
	    }
	}
    }

  XtSetArg(args[count], XmNresizable, TRUE); count++;
  XtSetArg(args[count], XmNborderWidth, 0); count++;

  XtSetArg(args[count], XmNhorizontalSpacing, 8); count++;
  XtSetArg(args[count], XmNverticalSpacing, 8); count++;

  *form = XmCreateForm(parent, "compForm", args, count);
  XtManageChild(*form);

  if (expand_this == FALSE)
    {
      XtSetArg(args[0], XmNresizable, FALSE);
      XtSetValues(*form, args, 1);
    }

  if (follow_expand == TRUE)
    {
      if (from_vert != NULLWidget)
	{
	  XtSetArg(args[0], XmNbottomAttachment, XmATTACH_WIDGET);
	  XtSetArg(args[1], XmNbottomWidget, *form);
	  XtSetValues(from_vert, args, 2);

	  XtSetArg(args[0], XmNtopAttachment, XmATTACH_NONE);
	  XtSetValues(*form, args, 1);
	}
      
      if (from_horiz != NULLWidget)
	{
	  XtSetArg(args[0], XmNrightAttachment, XmATTACH_WIDGET);
	  XtSetArg(args[1], XmNrightWidget, *form);
	  XtSetValues(from_vert, args, 2);

	  XtSetArg(args[0], XmNleftAttachment, XmATTACH_NONE);
	  XtSetValues(*form, args, 1);
	}
    }

  from_vert = from_horiz = NULLWidget;
  had_child_expand = FALSE;

  curr_sibling = widget_layout_ptr;
  while (curr_layout)
    {
      switch (curr_layout->ref)
	{
	case REF_layout:

	  *curr_sibling = WidgetTree_alloc();
	  (*curr_sibling)->left_siblings = NULLWidgetTree;
	  (*curr_sibling)->descendants = NULLWidgetTree;
	  
	  curr_descendant = &((*curr_sibling)->descendants);
	  
	  XLUmakeWindowFromTemplateAux
	    (objid,
	     tmplid, 
	     win_templates[curr_layout->reference.layout_ref_id]->layout,
	     curr_descendant,
	     *form,
	     &child_form,
	     from_vert,
	     from_horiz,
	     curr_layout->expand,
	     had_child_expand,
	     curr_layout->orient);

	  (*curr_sibling)->this_widget = child_form;
	  (*curr_sibling)->reference = REF_layout;
	  curr_sibling = &((*curr_sibling)->left_siblings);
	  
	  if (curr_orientation == horizontal)
	    from_horiz = child_form;
	  else
	    from_vert = child_form;

	  break;

	case REF_primitive:

	  curr_pw = makePrimitive(objid,
				  curr_layout,
				  *form,
				  from_vert,
				  from_horiz,
				  had_child_expand);

	  *curr_sibling = WidgetTree_alloc();
	  (*curr_sibling)->left_siblings = NULLWidgetTree;
	  (*curr_sibling)->descendants = NULLWidgetTree;
	  
	  (*curr_sibling)->this_widget = curr_pw;
	  (*curr_sibling)->reference = REF_primitive;
	  
	  curr_sibling = &((*curr_sibling)->left_siblings);
	  
	  if (curr_orientation == horizontal)
	    from_horiz = curr_pw;
	  else
	    from_vert = curr_pw;
	  
	  break;
	}
      
      if (curr_layout->expand == TRUE)
	had_child_expand = TRUE;

      if (curr_layout->next == NULLLayout && had_child_expand == TRUE)
	{
	  switch (curr_orientation)
	    {
	    case horizontal:
	      
	      XtSetArg(args[0], XmNrightAttachment, XmATTACH_FORM);
	      XtSetValues(from_horiz, args, 1);
	      break;
	      
	    case vertical:

	      XtSetArg(args[0], XmNbottomAttachment, XmATTACH_FORM);
	      XtSetValues(from_vert, args, 1);
	      break;
	    }
	}

      curr_layout->template_id = tmplid;
      curr_layout = curr_layout->next;
    }
} /* XLUmakeWindowFromTemplateAux */

/*****************************************************************************
 * doLayout()
 *
 * 
 *
 ****************************************************************************/

static void doLayout(template, widget_layout, parent)
     XLUlayoutObject template;
     WidgetTree widget_layout;
     Widget parent;
{
  orientation curr_orientation;
  XLUlayoutObject curr_layout = template, sub_layout;
  WidgetTree curr_widget_layout;
  Widget this_widget;
  Arg args[MAXARGS];
  int count;
  Dimension width, height;

  curr_widget_layout = widget_layout;
  curr_orientation = curr_layout->orient;

  while (curr_layout)
    {
      this_widget = curr_widget_layout->this_widget;

      if (curr_layout->ref == REF_layout)
	{
	  sub_layout =
	    win_templates[curr_layout->reference.layout_ref_id]->layout;
	  
	  doLayout
	    (sub_layout, curr_widget_layout->descendants, this_widget);
	  
	  if (sub_layout->orient == horizontal)
	    {
	      count = 0;
	      XtSetArg(args[count], XmNwidth, &width); count++;
	      XtGetValues(parent, args, count);
	      
	      count = 0;
	      XtSetArg(args[count], XmNwidth, (Dimension) width-2); count++;
	      XtSetValues(this_widget, args, count);
	    }
	  else
	    {
	      count = 0;
	      XtSetArg(args[count], XmNheight, &height); count++;
	      XtGetValues(parent, args, count);
	      
	      count = 0;
	      XtSetArg(args[count], XmNheight, (Dimension) height-2); count++;
	      XtSetValues(this_widget, args, count);
	    }
	}
      else
	layoutPrimitive(curr_layout->reference.primitive_id,
				 parent,
				 this_widget,
				 curr_layout,
				 curr_orientation);
      
      curr_widget_layout = curr_widget_layout->left_siblings;
      curr_layout = curr_layout->next;
    }
} /* doLayout */

/*****************************************************************************
 * layoutPrimitive()
 *
 * 
 *
 ****************************************************************************/

/* ARGSUSED */
static void layoutPrimitive(primid, parent, primitive, template, orient)
     int primid;
     Widget parent;
     Widget primitive;
     XLUlayoutObject template;
     orientation orient;
{
  Arg args[MAXARGS];
  
  extern char bitmap_directory[];

  if (template->expand == FALSE)
    {
      XtSetArg(args[0], XmNresizable, FALSE);
      XtSetValues(primitive, args, 1);
    }

  switch (primid)
    {
    case HELP:

      setHelp(primitive, template);
      break;
      
    default: /* default assumes a button */
      if (template->bitmap_file != NULLString)
	{
	  create_background_from_bitmap_file(primitive, template->bitmap_file);
	  
	  XtSetArg(args[1], XmNresizable, FALSE);
	  XtSetValues(primitive, args, 3);
	}
    }
} /* layoutPrimitive */

/*****************************************************************************
 * makePrimitive()
 *
 * 
 *
 ****************************************************************************/

static Widget makePrimitive(objid,
			    template,
			    parent,
			    from_vert,
			    from_horiz,
			    follow_expand)
     int objid;
     XLUlayoutObject template;
     Widget parent;
     Widget from_vert;
     Widget from_horiz;
     BOOLEAN follow_expand;
{
  Arg args[MAXARGS];
  int count;
  Widget widget = NULLWidget;
  int primid = template->reference.primitive_id;

  count = 0;

  XtSetArg(args[count], XmNinitialResourcesPersistent, FALSE); count++;

  XtSetArg(args[count], XmNleftOffset, 10); count++;
  XtSetArg(args[count], XmNtopOffset, 10); count++;

  if (from_vert != NULLWidget && follow_expand == FALSE)
    {
      XtSetArg(args[count], XmNtopAttachment, XmATTACH_WIDGET); count++;
      XtSetArg(args[count], XmNtopWidget, from_vert); count++;
    }

  if (from_vert == NULLWidget)
    {
      XtSetArg(args[count], XmNtopAttachment, XmATTACH_FORM); count++;
    }

  if (from_horiz != NULLWidget && follow_expand == FALSE)
    {
      XtSetArg(args[count], XmNleftAttachment, XmATTACH_WIDGET); count++;
      XtSetArg(args[count], XmNleftWidget, from_horiz); count++;
    }

  if (from_horiz == NULLWidget)
    {
      XtSetArg(args[count], XmNleftAttachment, XmATTACH_FORM); count++;
    }

  if (primid != BUTTON)
    {
      if (template->orient == horizontal)
	{
	  XtSetArg(args[count], XmNbottomAttachment, XmATTACH_FORM);count++;
	}
      else
	{
	  XtSetArg(args[count], XmNrightAttachment, XmATTACH_FORM);count++;
	}    
    }
  
  switch (primid)
    {
    case BUTTON:
      widget = XLUmakeButton(parent, objid, template, args, count);
      break;

    case ENTRY_LIST:
      widget = XLUmakeListViewer(parent, objid, template, args, count);
      break;

    case ERROR_VIEW:
      widget = XLUmakeErrorViewer(parent, objid, template, args, count);
      break;

    case SEARCH_LOG:
      widget = XLUmakeLogViewer(parent, objid, template, args, count);
      break;

    case HELP:
      widget = XLUmakeHelpViewer(parent, objid, template, args, count);
      break;

    case LOOKUP:
      widget = XLUmakeFormDialog(parent, objid, template, args, count);
      break;

    case MODIFY:
      widget = XLUmakeModifyDialog(parent, objid, template, args, count);
      break;

    case MOVETO:
      widget = XLUmakeUFNdialog(parent, objid, template, args, count);
      break;

    case ENTRY_DISPLAY:
      widget = XLUmakeEntryViewer(parent, objid, template, args, count);
      break;

    case STATUS:
      widget = XLUmakeStatusBar(parent, objid, template, args, count);
      break;

    case TITLE:
      widget = XLUmakeLabel(parent, objid, template, args, count);
      break;
    }
  
  if (follow_expand == TRUE)
    {
      if (from_vert != NULLWidget)
	{
	  XtSetArg(args[0], XmNbottomAttachment, XmATTACH_WIDGET);
	  XtSetArg(args[1], XmNbottomWidget, widget);
	  XtSetValues(from_vert, args, 2);

	  XtSetArg(args[0], XmNtopAttachment, XmATTACH_NONE);
	  XtSetValues(widget, args, 1);
	}

      if (from_horiz != NULLWidget)
	{
	  XtSetArg(args[0], XmNrightAttachment, XmATTACH_WIDGET);
	  XtSetArg(args[1], XmNrightWidget, widget);
	  XtSetValues(from_horiz, args, 2);

	  XtSetArg(args[0], XmNleftAttachment, XmATTACH_NONE);
	  XtSetValues(widget, args, 1);
	}
    }
  
  if (template->expand == FALSE)
    {
      XtSetArg(args[0], XmNresizable, FALSE);
      XtSetValues(widget, args, 1);
    }

  switch (primid)
    {
    case ENTRY_LIST:
      widget = XtParent(widget);
      break;
    }

  return widget;
} /* makePrimitive */

/*****************************************************************************
 * XLUgetPrimitiveTemplate()
 *
 * 
 *
 ****************************************************************************/

XLUlayoutObject XLUgetPrimitiveTemplate(objid, primid, pw)
     int objid;
     int primid;
     Widget pw;
{
  XLUwindowObject winobj;
  WidgetTree curr_widget;
  XLUlayoutObject curr_member;
  
  winobj = XLUgetWindowObject(objid);

  curr_member = win_templates[winobj->template_id]->layout;
  curr_widget = winobj->widget_layout->descendants;
  
  return XLUgetPrimitiveTemplateAux(curr_member, curr_widget, primid, pw);
} /* XLUgetPrimitiveTemplate */

/*****************************************************************************
 * XLUgetPrimitiveTemplateAux()
 *
 * 
 *
 ****************************************************************************/

static XLUlayoutObject XLUgetPrimitiveTemplateAux(curr_member,
				     curr_widget,
				     primid,
				     pw)
     XLUlayoutObject curr_member;
     WidgetTree curr_widget;
     int primid;
     Widget pw;
{
  XLUlayoutObject return_member;

  while (curr_member)
    {
      switch (curr_member->ref)
	{
	case REF_layout:

	  return_member = XLUgetPrimitiveTemplateAux
	       (win_templates[curr_member->reference.layout_ref_id]->layout,
		curr_widget->descendants,
		primid,
		pw);

	  if (return_member != NULLLayout)
	    return return_member;
      
	  break;
	  
	case REF_primitive:
	  if (curr_widget->this_widget == pw)
	    return curr_member;
	  
	  break;
	}
      
      curr_member = curr_member->next;
      curr_widget = curr_widget->left_siblings;
  }
  
  return NULLLayout;
} /* XLUgetPrimitiveTemplateAux */

/*****************************************************************************
 * XLUgetPrimitiveList()
 *
 * 
 *
 ****************************************************************************/

WidgetList XLUgetPrimitiveList(objid, primid)
     int objid;
     int primid;
{
  XLUwindowObject winobj;
  WidgetTree curr_widget;
  XLUlayoutObject curr_member;
  int num_widgets;
  WidgetList return_widgets;

  winobj = XLUgetWindowObject(objid);

  curr_member = win_templates[winobj->template_id]->layout;
  curr_widget = winobj->widget_layout->descendants;

  widget_count = 0;
  XLUgetPrimitiveListAux(primid, curr_member, curr_widget);

  if (widget_count == 0)
    return ((Widget *) NULL);
  
  return_widgets = (WidgetList)
    XtMalloc((unsigned) (sizeof(Widget) * (widget_count + 1)));

  for (num_widgets = 0; num_widgets < widget_count; num_widgets++)
    return_widgets[num_widgets] = widget_list[num_widgets];

  return_widgets[num_widgets] = NULLWidget;
  return return_widgets;
} /* XLUgetPrimitiveList */

/*****************************************************************************
 * XLUgetPrimitiveListAux()
 *
 * 
 *
 ****************************************************************************/

static void XLUgetPrimitiveListAux(primid, curr_member, curr_widget)
     int primid;
     XLUlayoutObject curr_member;
     WidgetTree curr_widget;
{
  while (curr_member != NULLLayout)
    {
      switch (curr_member->ref)
	{
	case REF_layout:

	  XLUgetPrimitiveListAux
	    (primid,
	     win_templates[curr_member->reference.layout_ref_id]->layout,
	     curr_widget->descendants);

	  break;
	  
	case REF_primitive:

	  if (curr_member->reference.primitive_id == primid)
	    widget_list[widget_count++] = curr_widget->this_widget;
	  
	  break;
	}
      
      curr_member = curr_member->next;
      curr_widget = curr_widget->left_siblings;
    }
} /* XLUgetPrimitiveListAux */

/*****************************************************************************
 * findNamedInstances()
 *
 * 
 *
 ****************************************************************************/

WidgetList findNamedInstances(objid, name)
     int objid;
     String name;
{
  XLUwindowObject winobj;
  WidgetTree curr_widget;
  XLUlayoutObject curr_member;
  int num_widgets;
  WidgetList return_widgets;

  winobj = XLUgetWindowObject(objid);

  curr_member = win_templates[winobj->template_id]->layout;
  curr_widget = winobj->widget_layout->descendants;

  widget_count = 0;
  findNamedInstancesAux(name, curr_member, curr_widget);

  if (widget_count == 0)
    return ((Widget *) NULL);
  
  return_widgets = (WidgetList)
    XtMalloc((unsigned) (sizeof(Widget) * (widget_count + 1)));

  for (num_widgets = 0; num_widgets < widget_count; num_widgets++)
    return_widgets[num_widgets] = widget_list[num_widgets];

  return_widgets[num_widgets] = NULLWidget;

  return return_widgets;
} /* findNamedInstances */

/*****************************************************************************
 * findNamedInstancesAux()
 *
 * 
 *
 ****************************************************************************/

static void findNamedInstancesAux(name, curr_member, curr_widget)
     String name;
     XLUlayoutObject curr_member;
     WidgetTree curr_widget;
{
  static String this_name;

  while (curr_member != NULLLayout)
    {
      switch (curr_member->ref)
	{
	case REF_layout:

	  findNamedInstancesAux
	    (name,
	     win_templates[curr_member->reference.layout_ref_id]->layout,
	     curr_widget->descendants);

	  break;
	  
	case REF_primitive:

	  this_name = curr_member->name;

	  if (this_name != NULLString && !lexequ(this_name, name))
	    widget_list[widget_count++] = curr_widget->this_widget;
	  
	  break;
	}
      
      curr_member = curr_member->next;
      curr_widget = curr_widget->left_siblings;
    }
} /* findNamedInstancesAux */
