#ifndef lint
static char *rcsid = "$Header$";
#endif

/* $Header$ */

/*****************************************************************************
 * 
 * - name.c -
 *
 * UFN-based search dialog. 
 *
 ****************************************************************************/

#include "name.h"
#include "query.h"

#include "defs.h"
#include "windows.h"
#include "widgets.h"
#include "objects.h"
#include "requests.h"
#include "callbacks.h"
#include "configure.h"
#include "actions.h"
#include "y.tab.h"
#include "layout.h"

#include "showlist.h"
#include "showentry.h"
#include "status.h"
#include "errview.h"
#include "xdutil.h"
#include "xutil.h"
#include "searchlog.h"
#include "update1.h"

/**************************************************
 *
 * External data..
 * 
 */

extern XLUobjectAction actions[];
extern int action_number; 
extern Display *dpy;

/*****************************************************************************
 * doMovetoAction()
 *
 * 
 *
 ****************************************************************************/

/* ARGSUSED */
void doMovetoAction(pw, objid, prim_id, action_id,actlist)
     Widget pw;
     int objid;
     int prim_id;
     int action_id;
     XLUactionList actlist;
{
  Arg args[MAXARGS];
  int count;
  int reqid;
  QyErrorCode srch_state;
  Widget dialog;
  String ufnval;
  stringCell select;
  int logwin_id;
  XLUoutputDef output;
  
  dialog = XtNameToWidget(pw, "movetoDialog");

  count = 0;
  XtSetArg(args[count], XmNvalue, &ufnval); count++;
  XtGetValues(dialog, args, count);

  select = GetFullSelection();

  srch_state = QyUFNResolve(NULLEntryList, ufnval, select, &reqid);
  switch (srch_state)
    {
    case QERR_bad_value_syntax:
      XBell(dpy, 40);
      XLUstatusMessage(objid, "The entry name entered is invalid!");

      callActionAux(pw,
		    objid,
		    prim_id,
		    action_id,
		    actlist->next);

      return;
      
    case QERR_internal_limit_reached:
      XBell(dpy, 40);
      XLUstatusMessage(objid, "Internal limit reached, cannot search!");

      callActionAux(pw,
		    objid,
		    prim_id,
		    action_id,
		    actlist->next);

      return;

    default:
      
      break;
    }

  /* Find log window, if supplied. */
  logwin_id = -1;
  for (output = actlist->outputs;
       output != (XLUoutputDef) NULL;
       output = output->next)
    {
      if (output->type == XLUlogTo)
	{
	  (void) XLUfindOutputObject(objid,
				     output->outputTmplID,
				     actlist->mode,
				     &logwin_id);

	  break;
	}
    }

  /* If a log output window has been found, make it the request owner. */
  if (logwin_id >= 0)
    objid = (int) logwin_id;

  XLUstatusMessage(objid, "Searching. Please Wait.");
  XLUrequestInvoked(reqid,
		  UFNAME,
		  gotMovetoResults,
		  gotMovetoReport,
		  pw,
		  objid,
		  prim_id,
		  action_id,
		  actlist);

  /* Log initial report */
  if (logwin_id >= 0)
    {
      LogUFNReport(logwin_id, reqid);
    }
} /* doMovetoAction */

/*****************************************************************************
 * gotMovetoReport()
 *
 * 
 *
 ****************************************************************************/

void gotMovetoReport(reqid, objid)
     int reqid;
     int objid;
{
  XLUwindowObject winobj;

  /* Check that the owner window is still on the desktop. */
  winobj = XLUgetWindowObject(objid);
  if (winobj == NULL)
    {
      /* If request owner has gone, abort the request. */
      XLUrequestComplete(reqid);
      QyAbortRequest(reqid);

      return;
    }

  /* Log the report. */
  LogUFNReport(objid, reqid);
} /* gotMovetoReport */

/*****************************************************************************
 * gotMovetoResults()
 *
 * 
 *
 ****************************************************************************/

/* ARGSUSED */
void gotMovetoResults(reqid, objid, action_id, actlist)
     int reqid;
     int objid;
     int action_id;
     XLUactionList actlist;
{
  QyUFNResult results;
  int child_id, tmpl_id, err_num;
  XLUactionType action_type;
  char status_message[LINESIZE], status_buf[LINESIZE];
  QyErrorList errors;

  /***************************************************
   *
   * Notify the user
   *
   */

  XLUstatusMessage(objid, "Results received.");
  XBell(dpy, 40);
  
  results = QyGetUFNResult(reqid);
 
  /***************************************************
   *
   * Will have to change this if poor
   * matches are to be queried.
   *
   */

  QyRequestComplete(reqid);

  /***************************************************
   *
   * Work out the error situation.
   *
   */
  
  for (err_num = 0, errors = results->errors;
       errors != NULLError;
       errors = errors->next, err_num++)
    ;

  XLUsetWindowErrors(objid, results->errors);

  if (err_num > 0)
    updateErrorViewer(objid);

  if (results->matchnum == 0 || results->type != QyUFNsuccess)
    {
      XBell(dpy, 40);

      if (err_num > 0)
	(void) strcpy(status_message, "Match Failed!");
      else
	(void) strcpy(status_message, "Nothing found!");
    }
  else
    for (; actlist != NULLActionList; actlist = actlist->next)
      {
	action_type = actlist->type;
	if (action_type !=AT_Activate)
	  continue;

	if (results->matchnum == 1)
	  {
	    (void) XLUgetOutputTemplateID(actlist->outputs,
					  XLUoneHit,
					  &tmpl_id);

	    (void) XLUfindOutputObject(objid,
				    tmpl_id,
				    actlist->mode,
				    &child_id);

	    (void) strcpy(status_message, "Made 1 good match");
	    XLUsetWindowEntry(child_id, results->matches);

	    displayEntryTemplate(NULLWidget, child_id, 0, 0, NULLActionList);
	    XLUwait(child_id);

	    displayCurrentEntry(NULLWidget, child_id, 0, 0, NULLActionList);
	    doPrimitiveSensitivity(child_id);
	  }
	else if (results->matchnum > 1)
	  {
	    (void) XLUgetOutputTemplateID(actlist->outputs,
					  XLUmanyHits,
					  &tmpl_id);
	    
	    (void) XLUfindOutputObject(objid,
				       tmpl_id,
				       actlist->mode,
				       &child_id);
	    
	    (void) sprintf(status_message,
			   "Made %d good matches",
			   results->matchnum);
	    
	    XLUsetWindowList(child_id, results->matches);
	    displayCurrentList(child_id);
	    
	    doPrimitiveSensitivity(child_id);
	  }
      }
  
  switch (err_num)
    {
    case 0:
      break;
      
    case 1:

      (void) strcat(status_message, "\n1 Error Occurred During Search");
      break;

    default:
      
      (void) sprintf(status_buf, "\n%d Errors Occurred During Search",err_num);
      (void) strcat(status_message, status_buf);
    }
  
  XLUstatusMessage(objid, status_message);
  XLUrequestComplete(reqid);

  QyFreeUFNResult(&results);
} /* gotMovetoResults */

