#ifndef _Xlu_objects_h_
#define _Xlu_objects_h_

#include "defs.h"
#include "windows.h"
#include "layout.h"

#include "dn_list.h"
#include "query.h"

#define MAXWINDOWS 64
#define MAXSTARTS 64

typedef struct widget_tree
{
  Widget this_widget;
  ReferenceType reference;

  struct widget_tree *descendants;
  struct widget_tree *left_siblings;
} *WidgetTree;

#define NULLWidgetTree (WidgetTree) NULL
#define WidgetTree_alloc() (WidgetTree) XtNew(struct widget_tree)

typedef struct child_object
{
  int child_object_id;
  struct child_object *left_siblings;
} child_list, *childList;
  
#define NULLChildList (childList) NULL
#define child_list_alloc() (childList) XtMalloc(sizeof(child_list))

/**************************************************
 *
 * This structure is used to keep track of all
 * top level windows.
 * 
 */

typedef struct XLUwindowObject
{
  int id;
  int template_id;

  BOOLEAN is_child;
  int parent_id;

  BOOLEAN is_kept;

  Widget parent_shell;
  WidgetTree widget_layout;

  childList children;
  int num_children;

  entryList current_entry;
  entryList current_list;
  entryList current_save_list;
  QyErrorList current_errors;
} *XLUwindowObject;

#define NULLWinObject (XLUwindowObject) NULL
#define object_alloc() (XLUwindowObject) \
	XtMalloc(sizeof(struct XLUwindowObject))

/***************************************************
 *
 * Global functions.
 *
 */

void XLUsetWindowEntry();
void XLUsetWindowErrors();

void XLUsetWindowList();
void XLUsetWindowSaveList();

BOOLEAN XLUisWaiting();
void XLUwait();

void CreateStartupWindows();
void free_child_list();

void free_widget_tree();

int XLUfindUnkeptChild();
int XLUfindUnkeptObject();

int XLUgetWindowObjectId();

Widget XLUmakeWindowObject();
XLUwindowObject XLUgetWindowObject();
XLUwindowObject XLUnewWindowObject();
void XLUcloseWindowObject();

#endif /* _Xlu_objects_h_ */



