/**********************************************************************
 *
 * - select.c -
 * 
 * Copy MUA related attributes (OR address, telephone, DN and name)
 * into an X selection buffer.
 * 
 **********************************************************************/

#include "defs.h"
#include "select.h"
#include "objects.h"
#include "actions.h"
#include "xdutil.h"
#include "status.h"
#include "mua_selection.h"
#include <X11/Xatom.h>

#ifdef USE_PP
#include "or.h"
#endif

/**************************************************
 *
 * Local data.
 * 
 */

char orstr[LINESIZE];
char phonestr[LINESIZE];
char namestr[LINESIZE];
char dnstr[LINESIZE];
static String select_str = NULLString;

Widget selw = NULLWidget; 

/**************************************************
 *
 * Local functions
 * 
 */

static int LoseDUA();
static int ConvertDUA();
void GetORAddress();
static void GetTelephoneNumber();

/*****************************************************************************
 * doSelectAction()
 *
 * Take ownership
 *
 ****************************************************************************/

void doSelectAction(pw, objid, primid, actid, actlist)
     Widget pw;
     int objid;
     int primid;
     int actid;
     XLUactionList actlist;
{
  entryList centry;
  char buf[LINESIZE], msgbuf[LINESIZE];
  XLUwindowObject winobj;

  /* Get current entry */
  winobj = XLUgetWindowObject(objid);

  centry = winobj->current_entry;
  if (centry == NULLEntryList)
    return;

  /* Get mail address value */
  GetORAddress(centry->entry, buf);

  if (isnull(*buf))
    /* Can\'t continue if no mail address supplied */
    return;

  /* Clear current selection */
  (void) LoseDUA((caddr_t) NULL);

  /* Save mail address */
  (void) strcpy(orstr, buf);

  /* Get and save supporting data (=phone, DN and friendly name */
  GetTelephoneNumber(centry->entry, phonestr);
  (void) strcpy(dnstr, centry->string_dn);
  (void) QyPrintFriendlyDN(dnstr, namestr);

  /* Set and own the selection */
  (void) sprintf(buf, "%s\n%s\n%s\n%s", namestr, dnstr, phonestr, orstr);

  select_str = XtNewString(buf);
  selw = pw;

  (void) OwnMUA(pw, ConvertDUA, NULL, (caddr_t) select_str);
} /* doSelectAction */

/*****************************************************************************
 * ConvertDUA()
 *
 * 
 *
 ****************************************************************************/

static int ConvertDUA(data, nameptr, phoneptr, dnptr, orptr)
     caddr_t data;
     String *nameptr;
     String *phoneptr;
     String *dnptr;
     String *orptr;
{
  String start, end;
  
  *nameptr = namestr;
  *phoneptr = phonestr;
  *dnptr = dnstr;
  *orptr = orstr;

  end = index((start = (String) data), '\n');
  *end = '\0';
  (void) strcpy(namestr, start);
  *end++ = '\n';

  end = index((start = end), '\n');
  *end = '\0';
  (void) strcpy(dnstr, start);
  *end++ = '\n';

  end = index((start = end), '\n');
  *end = '\0';
  (void) strcpy(phonestr, start);
  *end++ = '\n';

  start = end;
  (void) strcpy(orstr, start);

  return 1;
} /* ConvertDUA */

/*****************************************************************************
 * LoseDUA()
 *
 * The selection has been lost. Clean up.
 *
 ****************************************************************************/

static int LoseDUA(data)
  caddr_t data;
{
  if (select_str != NULLString)
    XtFree(select_str);

  select_str = NULLString;
} /* LoseDUA */

/*****************************************************************************
 * GetORAddress()
 *
 * Retrieve an OR address from the given entry and copy it into a
 * global string list.
 *
 ****************************************************************************/

void GetORAddress(attrs, orbuf)
     QyAttrValList attrs;
     String orbuf;
{
  stringCell select, mailstr, orstrlist;
#ifdef USE_PP
  OR_ptr or, compor;
#endif /* USE_PP */

  *orbuf = '\0';

  select = NULLStrCell;
  SeqAddString("0.9.2342.19200300.100.1.2", &select);

  orstrlist = XLUgetEntryValues(attrs, select);
  SeqFree(&select);

  if (orstrlist != NULLStrCell)
    {
      (void) strcpy(orbuf, orstrlist->string);
      SeqFree(&orstrlist);
    }

#ifdef USE_PP
  if (isnull(*orbuf))
    /* No OR address found, try to convert rfc822 mailbox, if found */
    {
      select = NULLStrCell;
      SeqAddString("0.9.2342.19200300.100.1.3", &select);

      mailstr = XLUgetEntryValues(attrs, select);
      SeqFree(&select);
      
      if (mailstr != NULLStrCell && or_rfc2or(mailstr->string, &or) == OK)
	{
	  compor = or_default(or);
	  or_or2std(compor, orbuf, 1);

	  or_free(or);
	  or_free(compor);

	  SeqFree(&mailstr);
	}
    }
#endif /* USE_PP */
} /* GetORAddress */

/*****************************************************************************
 * GetTelephoneNumber()
 *
 * 
 *
 ****************************************************************************/

static void GetTelephoneNumber(attrs, phonebuf)
     QyAttrValList attrs;
     String phonebuf;
{
  stringCell select, phonelist;

  *phonebuf = '\0';

  select = NULLStrCell;
  SeqAddString("2.5.4.20", &select);

  phonelist = XLUgetEntryValues(attrs, select);
  SeqFree(&select);

  if (phonelist != NULLStrCell)
    {
      (void) strcpy(phonebuf, phonelist->string);
      SeqFree(&phonelist);
    }
} /* GetTelephoneNumber */
